import argparse

# from pathlib import Path # TODOLATER:

# Local imports
from .app import app


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--port", type=int, default=8050)
    parser.add_argument("-d", "--debug", type=bool, default=True)
    # parser.add_argument("-r", "--reset", type=bool, default=False)
    args = parser.parse_args()

    # if args.reset:
    #     # Reset the state
    #     print("Resetting the state...")
    #     Path("state.json").unlink(missing_ok=True)

    app.run(debug=args.debug, port=args.port)



from dash import dcc, html

# Local imports
from qimchi.state import (
    # Line & marker options
    DEFAULT_LINE_MODE,
    DEFAULT_LINE_COLOR,
    DEFAULT_LINE_WIDTH,
    DEFAULT_LINE_OPACITY,
    DEFAULT_LINE_DASH,
    DEFAULT_LINE_SHAPE,
    DEFAULT_SPLINE_SMOOTHING,
    DEFAULT_MARKER_COLOR,
    DEFAULT_MARKER_SIZE,
    DEFAULT_MARKER_SYMBOL,
    DEFAULT_MARKER_OPACITY,
    LINE_COLOR_OPTS,
    LINE_DASH_OPTS,
    LINE_MODE_OPTS,
    LINE_SHAPE_OPTS,
    MARKER_COLOR_OPTS,
    MARKER_SYMBOL_OPTS,
)


def appearance_line_dropdown(fig_id: str) -> html.Div:
    return dcc.Dropdown(
        id={
            "index": fig_id,
            "key": "line",
            "setting": "mode",
        },
        options=LINE_MODE_OPTS,
        value=DEFAULT_LINE_MODE,
        placeholder="Line Mode",
        clearable=False,
        persistence=True,
        persistence_type="local",
    )


def appearance_line_options(fig_id: str) -> html.Div:
    return html.Div(
        id={
            "setfor": "line",
            "index": fig_id,
        },
        children=[
            html.Div(
                [
                    dcc.Dropdown(
                        id={
                            "index": fig_id,
                            "key": "line",
                            "setting": "color",
                        },
                        options=LINE_COLOR_OPTS,
                        value=DEFAULT_LINE_COLOR,
                        placeholder="Line Color",
                        clearable=False,
                        persistence=True,
                        persistence_type="local",
                        style={"width": "100%", "margin": "2px"},
                    ),
                    dcc.Dropdown(
                        id={
                            "index": fig_id,
                            "key": "line",
                            "setting": "dash",
                        },
                        options=LINE_DASH_OPTS,
                        value=DEFAULT_LINE_DASH,
                        placeholder="Line Dash",
                        clearable=False,
                        persistence=True,
                        persistence_type="local",
                        style={"width": "100%", "margin": "2px"},
                    ),
                    dcc.Dropdown(
                        id={
                            "index": fig_id,
                            "key": "line",
                            "setting": "shape",
                        },
                        options=LINE_SHAPE_OPTS,
                        value=DEFAULT_LINE_SHAPE,
                        placeholder="Line Shape",
                        clearable=False,
                        persistence=True,
                        persistence_type="local",
                        style={"width": "100%", "margin": "2px"},
                    ),
                ],
                className="column is-full is-flex px-0",
            ),
            html.Div(
                [
                    html.Div(
                        [
                            html.Label(
                                "Line Width: ",
                                className="column is-one-quarter",
                                style={"fontWeight": "bold"},
                            ),
                            dcc.Slider(
                                min=0.0,
                                max=20.0,
                                step=0.5,
                                marks={0: "0", 20: "20"},
                                value=DEFAULT_LINE_WIDTH,
                                id={
                                    "index": fig_id,
                                    "key": "line",
                                    "setting": "width",
                                },
                                persistence=True,
                                persistence_type="local",
                                className="column is-three-quarters mx-0 my-2 p-3",
                            ),
                        ],
                        className="column is-flex is-one-third",
                    ),
                    html.Div(
                        [
                            html.Label(
                                "Line Opacity: ",
                                className="column is-one-quarter",
                                style={"fontWeight": "bold"},
                            ),
                            dcc.Slider(
                                min=0.0,
                                max=1.0,
                                step=0.05,
                                marks={0: "0", 1: "1"},
                                value=DEFAULT_LINE_OPACITY,
                                id={
                                    "index": fig_id,
                                    "key": "line",
                                    "setting": "opacity",
                                },
                                persistence=True,
                                persistence_type="local",
                                className="column is-three-quarters mx-0 my-2 p-3",
                            ),
                        ],
                        className="column is-flex is-one-third",
                    ),
                    html.Div(
                        [
                            html.Label(
                                "Spline Smoothing: ",
                                className="column is-one-quarter",
                                style={"fontWeight": "bold"},
                            ),
                            dcc.Slider(
                                min=0.0,
                                max=1.3,
                                step=0.05,
                                marks={0: "0", 1.3: "1.3"},
                                value=DEFAULT_SPLINE_SMOOTHING,
                                id={
                                    "index": fig_id,
                                    "key": "line",
                                    "setting": "smoothing",
                                },
                                persistence=True,
                                persistence_type="local",
                                className="column is-three-quarters mx-0 my-2 p-3",
                            ),
                        ],
                        className="column is-flex is-one-third",
                    ),
                ],
                className="column is-full is-flex p-0",
            ),
        ],
        # Always visible
        className="column is-full is-flex-wrap-wrap p-0",
    )


def appearance_marker_options(fig_id: str) -> html.Div:
    return html.Div(
        id={
            "setfor": "marker",
            "index": fig_id,
        },
        children=[
            html.Div(
                [
                    html.Div(
                        [
                            dcc.Dropdown(
                                id={
                                    "index": fig_id,
                                    "key": "marker",
                                    "setting": "color",
                                },
                                options=MARKER_COLOR_OPTS,
                                value=DEFAULT_MARKER_COLOR,
                                placeholder="Marker Color",
                                clearable=False,
                                persistence=True,
                                persistence_type="local",
                                # NOTE: className="column is-half" refuses to work here
                                style={"width": "100%", "margin": "2px"},
                            ),
                            dcc.Dropdown(
                                id={
                                    "index": fig_id,
                                    "key": "marker",
                                    "setting": "symbol",
                                },
                                options=MARKER_SYMBOL_OPTS,
                                value=DEFAULT_MARKER_SYMBOL,
                                placeholder="Marker Symbol",
                                clearable=False,
                                persistence=True,
                                persistence_type="local",
                                # NOTE: className="column is-half" refuses to work here
                                style={"width": "100%", "margin": "2px"},
                            ),
                        ],
                        className="column is-flex is-full px-0",
                    ),
                    html.Div(
                        [
                            html.Div(
                                [
                                    html.Label(
                                        "Marker Size: ",
                                        className="column is-one-fifth",
                                        style={"fontWeight": "bold"},
                                    ),
                                    dcc.Slider(
                                        min=0.0,
                                        max=20.0,
                                        step=0.5,
                                        marks={0: "0", 20: "20"},
                                        value=DEFAULT_MARKER_SIZE,
                                        id={
                                            "index": fig_id,
                                            "key": "marker",
                                            "setting": "size",
                                        },
                                        persistence=True,
                                        persistence_type="local",
                                        className="column is-four-fifths mx-0 my-2 p-3",
                                    ),
                                ],
                                className="column is-flex is-half p-0",
                            ),
                            html.Div(
                                [
                                    html.Label(
                                        "Marker Opacity: ",
                                        className="column is-one-fifth",
                                        style={"fontWeight": "bold"},
                                    ),
                                    dcc.Slider(
                                        min=0.0,
                                        max=1.0,
                                        step=0.05,
                                        marks={0: "0", 1: "1"},
                                        value=DEFAULT_MARKER_OPACITY,
                                        id={
                                            "index": fig_id,
                                            "key": "marker",
                                            "setting": "opacity",
                                        },
                                        persistence=True,
                                        persistence_type="local",
                                        className="column is-four-fifths mx-0 my-2 p-3",
                                    ),
                                ],
                                className="column is-flex is-half p-0",
                            ),
                        ],
                        className="column is-flex is-full",
                    ),
                ],
                className="column is-flex is-flex-wrap-wrap is-full p-0",
            )
        ],
        # Initially hidden
        className="column is-full is-flex-wrap-wrap p-0",
    )



import json
from ast import literal_eval
from datetime import datetime

from dash import Input, Output, callback, html

# Local imports
from qimchi.components.utils import parse_data as parse


def metadata_viewer() -> html.P:
    """
    Metadata viewer

    Returns:
        dash.html.P: Paragraph element, displaying all the metadata relevant to the uploaded data

    """
    return html.P(
        "", className="column is-full has-background-light", id="metadata-viewer"
    )


def _render_tree(data):
    """
    Recursive function to render collapsible items
    """
    if isinstance(data, dict):
        return html.Ul(
            [
                html.Li(
                    # If the value is an empty list, empty string, empty dict, or None, render as "key: value" without a collapsible arrow
                    (
                        html.Span(
                            [
                                html.Strong(f"{key}", style={"color": "black"}),
                                f": {value}",
                            ]
                        )
                        if value in [None, [], "", {}]
                        else (
                            html.Span(
                                [
                                    html.Strong(f"{key}", style={"color": "black"}),
                                    f": {value}",
                                ]
                            )
                            if not isinstance(value, dict)
                            # Otherwise, make it collapsible, if it's a dictionary
                            else html.Details(
                                [
                                    html.Summary(
                                        html.Strong(key, style={"color": "black"}),
                                        style={"cursor": "pointer"},
                                    ),
                                    _render_tree(
                                        value
                                    ),  # NOTE: Recursively render nested dicts
                                ]
                            )
                        )
                    ),
                    style={
                        "listStyleType": "none",
                        "marginLeft": "20px",
                    },  # No bullets, indented
                )
                for key, value in data.items()
            ],
            style={"paddingLeft": "0px"},
        )
    else:
        # Render non-dict values directly as strings in a span
        return html.Span(str(data))


@callback(
    Output("metadata-viewer", "children"),
    Input("upload-data", "data"),
)
def update_metadata_viewer(contents: dict) -> list:
    """
    Callback to update metadata viewer

    Args:
        contents (dict): Dict representation of xarray.Dataset

    Returns:
        list: Metadata list to be displayed in the metadata viewer

    """
    if contents is None:
        return "No Data Uploaded"

    data = parse(contents)
    metadata = data.attrs
    dt = datetime.fromisoformat(metadata["Timestamp"])
    meta = []

    # Do not populate these keys in the metadata viewer
    forbidden_keys = [
        "Requirements",
        "Code Archive",
    ]
    for key in metadata:
        if key in forbidden_keys:
            continue

        meta.append(html.B(f"{key.capitalize()}: "))
        if key == "Timestamp":
            meta.append(dt.strftime("%Y-%m-%d %H:%M:%S"))
            meta.append(html.Br())

        elif key == "Cryostat":
            data = metadata[key].capitalize()
            meta.append(data)
            meta.append(html.Br())

        elif key == "independent_parameter_instruments":
            data = metadata[key]
            data = literal_eval(
                data
            )  # NOTE: The dicts are stored as strings in the list
            for dct in data:
                # NOTE: Convert single quotes to double quotes and None to "None" to make the string JSON serializable
                dct = dct.replace("'", '"')
                dct = dct.replace("None", '"None"')

                # Load the now JSON-serializable string
                dct = json.loads(dct)
                meta.append(
                    html.Details(
                        [
                            html.Summary(
                                html.Strong(
                                    f"{dct['name'].upper()}", style={"color": "black"}
                                ),
                                style={"cursor": "pointer"},
                            ),
                            _render_tree(dct),
                        ]
                    ),
                )
        else:
            meta.append(metadata[key])
            meta.append(html.Br())

    return meta



import xarray as xr
from dash.exceptions import PreventUpdate


def parse_data(data: dict) -> xr.Dataset:
    """
    Utility function to parse data into xarray dataset

    Args:
        data (dict): Dictionary containing the data to be parsed

    Returns:
        xr.Dataset: Parsed xarray dataset

    """
    if data is None:
        raise PreventUpdate
    return xr.Dataset.from_dict(data)



import logging
from rich.logging import RichHandler

logging.basicConfig(
    level=logging.NOTSET,
    format="%(message)s",
    handlers=[RichHandler()],
)

# Qimchi Logger
logger = logging.getLogger(__name__)
